@@ -4,9 +4,9 @@ from typing import Any, Dict
 
 import click
 from flask import Flask, current_app
-from flask.cli import FlaskGroup, with_appcontext
+from flask.cli import FlaskGroup, ScriptInfo, with_appcontext
 
-from alerta.app import config, db, key_helper, qb
+from alerta.app import config, create_app, db, key_helper, qb
 from alerta.auth.utils import generate_password_hash
 from alerta.models.enums import Scope
 from alerta.models.key import ApiKey
@@ -15,7 +15,7 @@ from alerta.settings import DEFAULT_ADMIN_ROLE
 from alerta.version import __version__
 
 
-def create_app(config_override: Dict[str, Any] = None, environment: str = None) -> Flask:
+def _create_app(config_override: Dict[str, Any] = None, environment: str = None) -> Flask:
     app = Flask(__name__)
     app.config['ENVIRONMENT'] = environment
     config.init_app(app)
@@ -28,13 +28,19 @@ def create_app(config_override: Dict[str, Any] = None, environment: str = None)
     return app
 
 
-@click.group(cls=FlaskGroup, create_app=create_app, add_version_option=False)
+@click.group(cls=FlaskGroup, add_version_option=False)
 @click.version_option(version=__version__)
-def cli():
+@click.pass_context
+def cli(ctx):
     """
     Management command-line tool for Alerta server.
     """
-    pass
+    if ctx.invoked_subcommand in ['routes', 'run', 'shell']:
+        # Load HTTP endpoints for standard Flask commands
+        ctx.obj = ScriptInfo(create_app=create_app)
+    else:
+        # Do not load HTTP endpoints for management commands
+        ctx.obj = ScriptInfo(create_app=_create_app)
 
 
 @cli.command('key', short_help='Create an admin API key')
@@ -210,7 +216,3 @@ def users():
         else:
             if user:
                 click.echo('{} {}'.format(user.id, user.login))
-
-
-if __name__ == '__main__':
-    cli()
