@@ -5,6 +5,7 @@ from flask import current_app
 from pymongo import ASCENDING, TEXT, MongoClient, ReturnDocument
 from pymongo.errors import ConnectionFailure
 
+from alerta.app import alarm_model
 from alerta.database.base import Database
 from alerta.exceptions import NoCustomerMatch
 from alerta.models.enums import ADMIN_SCOPES
@@ -23,6 +24,7 @@ class Backend(Database):
 
         db = self.connect()
         self._create_indexes(db)
+        self._update_lookups(db)
 
     def connect(self):
         self.client = MongoClient(self.uri)
@@ -51,6 +53,21 @@ class Backend(Database):
         db.groups.create_index([('name', ASCENDING)], unique=True)
         db.metrics.create_index([('group', ASCENDING), ('name', ASCENDING)], unique=True)
 
+    @staticmethod
+    def _update_lookups(db):
+        for severity, code in alarm_model.Severity.items():
+            db.codes.update(
+                {'severity': severity},
+                {'$set': {'severity': severity, 'code': code}},
+                upsert=True
+            )
+        for status, state in alarm_model.Status.items():
+            db.states.update(
+                {'status': status},
+                {'$set': {'status': status, 'state': state}},
+                upsert=True
+            )
+
     @property
     def name(self):
         return self.get_db().name
@@ -467,7 +484,29 @@ class Backend(Database):
 
     def get_alerts(self, query=None, page=None, page_size=None):
         query = query or Query()
-        return self.get_db().alerts.find(query.where, sort=query.sort).skip((page - 1) * page_size).limit(page_size)
+        pipeline = [
+            {'$lookup': {
+                'from': 'codes',
+                'localField': 'severity',
+                'foreignField': 'severity',
+                'as': 'fromCodes'
+            }},
+            {'$replaceRoot': {'newRoot': {'$mergeObjects': [{'$arrayElemAt': ['$fromCodes', 0]}, '$$ROOT']}}},
+            {'$project': {'fromCodes': 0}},
+            {'$lookup': {
+                'from': 'states',
+                'localField': 'status',
+                'foreignField': 'status',
+                'as': 'fromStates'
+            }},
+            {'$replaceRoot': {'newRoot': {'$mergeObjects': [{'$arrayElemAt': ['$fromStates', 0]}, '$$ROOT']}}},
+            {'$project': {'fromStates': 0}},
+            {'$match': query.where},
+            {'$sort': {k: v for k, v in query.sort}},
+            {'$skip': (page - 1) * page_size},
+            {'$limit': page_size}
+        ]
+        return self.get_db().alerts.aggregate(pipeline)
 
     def get_alert_history(self, alert, page=None, page_size=None):
         query = {
@@ -754,13 +793,14 @@ class Backend(Database):
         for r in response_status:
             status_count[r['_id']['environment']].append((r['_id']['status'], r['count']))
 
+        environments = self.get_db().alerts.find().distinct('environment')
         return [
             {
                 'environment': env,
                 'severityCounts': dict(severity_count[env]),
                 'statusCounts': dict(status_count[env]),
                 'count': sum(t[1] for t in severity_count[env])
-            } for env in severity_count]
+            } for env in environments]
 
     # SERVICES
 
@@ -791,14 +831,20 @@ class Backend(Database):
         for r in response_status:
             status_count[(r['_id']['environment'], r['_id']['service'])].append((r['_id']['status'], r['count']))
 
+        pipeline = [
+            {'$unwind': '$service'},
+            {'$group': {'_id': {'environment': '$environment', 'service': '$service'}}},
+            {'$limit': topn}
+        ]
+        services = list(self.get_db().alerts.aggregate(pipeline))
         return [
             {
-                'environment': env,
-                'service': svc,
-                'severityCounts': dict(severity_count[(env, svc)]),
-                'statusCounts': dict(status_count[(env, svc)]),
-                'count': sum(t[1] for t in severity_count[(env, svc)])
-            } for env, svc in severity_count]
+                'environment': svc['_id']['environment'],
+                'service': svc['_id']['service'],
+                'severityCounts': dict(severity_count[(svc['_id']['environment'], svc['_id']['service'])]),
+                'statusCounts': dict(status_count[(svc['_id']['environment'], svc['_id']['service'])]),
+                'count': sum(t[1] for t in severity_count[(svc['_id']['environment'], svc['_id']['service'])])
+            } for svc in services]
 
     # ALERT GROUPS
 
@@ -1617,7 +1663,7 @@ class Backend(Database):
 
     # HOUSEKEEPING
 
-    def housekeeping(self, expired_threshold, info_threshold):
+    def get_expired(self, expired_threshold, info_threshold):
         # delete 'closed' or 'expired' alerts older than "expired_threshold" hours
         # and 'informational' alerts older than "info_threshold" hours
 
@@ -1632,15 +1678,15 @@ class Backend(Database):
 
         # get list of alerts to be newly expired
         pipeline = [
-            {'$project': {
-                'event': 1, 'status': 1, 'lastReceiveId': 1, 'timeout': 1,
+            {'$addFields': {
                 'expireTime': {'$add': ['$lastReceiveTime', {'$multiply': ['$timeout', 1000]}]}}
              },
             {'$match': {'status': {'$nin': ['expired', 'ack', 'shelved']}, 'expireTime': {'$lt': datetime.utcnow()}, 'timeout': {
                 '$ne': 0}}}
         ]
-        has_expired = [(r['_id'], r['event'], r['lastReceiveId']) for r in self.get_db().alerts.aggregate(pipeline)]
+        return self.get_db().alerts.aggregate(pipeline)
 
+    def get_timeout(self):
         # get list of alerts to be unshelved
         pipeline = [
             {'$match': {'status': 'shelved'}},
@@ -1650,21 +1696,12 @@ class Backend(Database):
                 'history.status': 'shelved'
             }},
             {'$sort': {'history.updateTime': -1}},
-            {'$group': {
-                '_id': '$_id',
-                'event': {'$first': '$event'},
-                'lastReceiveId': {'$first': '$lastReceiveId'},
-                'updateTime': {'$first': '$history.updateTime'},
-                'timeout': {'$first': '$timeout'}
-            }},
-            {'$project': {
-                'event': 1,
-                'lastReceiveId': 1,
+            {'$addFields': {
                 'expireTime': {'$add': ['$updateTime', {'$multiply': ['$timeout', 1000]}]}
             }},
             {'$match': {'expireTime': {'$lt': datetime.utcnow()}, 'timeout': {'$ne': 0}}}
         ]
-        unshelve = [(r['_id'], r['event'], r['lastReceiveId']) for r in self.get_db().alerts.aggregate(pipeline)]
+        unshelve = self.get_db().alerts.aggregate(pipeline)
 
         # get list of alerts to be unack'ed
         pipeline = [
@@ -1675,20 +1712,11 @@ class Backend(Database):
                 'history.status': 'ack'
             }},
             {'$sort': {'history.updateTime': -1}},
-            {'$group': {
-                '_id': '$_id',
-                'event': {'$first': '$event'},
-                'lastReceiveId': {'$first': '$lastReceiveId'},
-                'updateTime': {'$first': '$history.updateTime'},
-                'timeout': {'$first': '$timeout'}
-            }},
-            {'$project': {
-                'event': 1,
-                'lastReceiveId': 1,
+            {'$addFields': {
                 'expireTime': {'$add': ['$updateTime', {'$multiply': ['$timeout', 1000]}]}
             }},
             {'$match': {'expireTime': {'$lt': datetime.utcnow()}, 'timeout': {'$ne': 0}}}
         ]
-        unack = [(r['_id'], r['event'], r['lastReceiveId']) for r in self.get_db().alerts.aggregate(pipeline)]
+        unack = self.get_db().alerts.aggregate(pipeline)
 
-        return has_expired, unshelve + unack
+        return list(unshelve) + list(unack)
