@@ -7,6 +7,7 @@ from flask import current_app
 from psycopg2.extensions import AsIs, adapt, register_adapter
 from psycopg2.extras import Json, NamedTupleCursor, register_composite
 
+from alerta.app import alarm_model
 from alerta.database.base import Database
 from alerta.exceptions import NoCustomerMatch
 from alerta.models.enums import ADMIN_SCOPES
@@ -380,11 +381,21 @@ class Backend(Database):
 
     def get_alerts(self, query=None, page=None, page_size=None):
         query = query or Query()
+        join = ''
+        if 's.code' in query.sort:
+            join += 'JOIN (VALUES {}) AS s(sev, code) ON alerts.severity = s.sev '.format(
+                ', '.join(("('{}', {})".format(k, v) for k, v in alarm_model.Severity.items()))
+            )
+        if 'st.state' in query.sort:
+            join += 'JOIN (VALUES {}) AS st(sts, state) ON alerts.status = st.sts '.format(
+                ', '.join(("('{}', '{}')".format(k, v) for k, v in alarm_model.Status.items()))
+            )
         select = """
-            SELECT * FROM alerts
+            SELECT *
+              FROM alerts {join}
              WHERE {where}
           ORDER BY {order}
-        """.format(where=query.where, order=query.sort or 'last_receive_time')
+        """.format(join=join, where=query.where, order=query.sort or 'last_receive_time')
         return self._fetchall(select, query.vars, limit=page_size, offset=(page - 1) * page_size)
 
     def get_alert_history(self, alert, page=None, page_size=None):
@@ -581,7 +592,7 @@ class Backend(Database):
 
         severity_count = defaultdict(list)
         status_count = defaultdict(list)
-        total_count = dict()
+        total_count = defaultdict(int)
 
         for row in result:
             if row.severity and not row.status:
@@ -591,13 +602,15 @@ class Backend(Database):
             if not row.severity and not row.status:
                 total_count[row.environment] = row.count
 
+        select = """SELECT DISTINCT environment FROM alerts"""
+        environments = self._fetchall(select, {})
         return [
             {
-                'environment': env,
-                'severityCounts': dict(severity_count[env]),
-                'statusCounts': dict(status_count[env]),
-                'count': total_count[env]
-            } for env in severity_count]
+                'environment': e.environment,
+                'severityCounts': dict(severity_count[e.environment]),
+                'statusCounts': dict(status_count[e.environment]),
+                'count': total_count[e.environment]
+            } for e in environments]
 
     # SERVICES
 
@@ -612,7 +625,7 @@ class Backend(Database):
 
         severity_count = defaultdict(list)
         status_count = defaultdict(list)
-        total_count = dict()
+        total_count = defaultdict(int)
 
         for row in result:
             if row.severity and not row.status:
@@ -622,14 +635,16 @@ class Backend(Database):
             if not row.severity and not row.status:
                 total_count[(row.environment, row.svc)] = row.count
 
+        select = """SELECT DISTINCT environment, svc FROM alerts, UNNEST(service) svc"""
+        services = self._fetchall(select, {})
         return [
             {
-                'environment': env,
-                'service': svc,
-                'severityCounts': dict(severity_count[(env, svc)]),
-                'statusCounts': dict(status_count[(env, svc)]),
-                'count': total_count[(env, svc)]
-            } for env, svc in severity_count]
+                'environment': s.environment,
+                'service': s.svc,
+                'severityCounts': dict(severity_count[(s.environment, s.svc)]),
+                'statusCounts': dict(status_count[(s.environment, s.svc)]),
+                'count': total_count[(s.environment, s.svc)]
+            } for s in services]
 
     # ALERT GROUPS
 
@@ -640,7 +655,12 @@ class Backend(Database):
             WHERE {where}
             GROUP BY environment, "group"
         """.format(where=query.where)
-        return [{'environment': g.environment, 'group': g.group, 'count': g.count} for g in self._fetchall(select, query.vars, limit=topn)]
+        return [
+            {
+                'environment': g.environment,
+                'group': g.group,
+                'count': g.count
+            } for g in self._fetchall(select, query.vars, limit=topn)]
 
     # ALERT TAGS
 
@@ -1296,7 +1316,7 @@ class Backend(Database):
 
     # HOUSEKEEPING
 
-    def housekeeping(self, expired_threshold, info_threshold):
+    def get_expired(self, expired_threshold, info_threshold):
         # delete 'closed' or 'expired' alerts older than "expired_threshold" hours
         # and 'informational' alerts older than "info_threshold" hours
         if expired_threshold:
@@ -1317,46 +1337,48 @@ class Backend(Database):
 
         # get list of alerts to be newly expired
         select = """
-            SELECT id, event, last_receive_id
+            SELECT *
               FROM alerts
              WHERE status NOT IN ('expired','ack','shelved') AND COALESCE(timeout, {timeout})!=0
                AND (last_receive_time + INTERVAL '1 second' * timeout) < NOW() at time zone 'utc'
         """.format(timeout=current_app.config['ALERT_TIMEOUT'])
-        has_expired = self._fetchall(select, {})
 
+        return self._fetchall(select, {})
+
+    def get_timeout(self):
         # get list of alerts to be unshelved
         select = """
         WITH shelved AS (
-            SELECT DISTINCT ON (a.id) a.id, a.event, a.last_receive_id, h.update_time, a.timeout
+            SELECT DISTINCT ON (a.id) a.*
               FROM alerts a, UNNEST(history) h
              WHERE a.status='shelved'
                AND h.type='shelve'
                AND h.status='shelved'
           ORDER BY a.id, h.update_time DESC
         )
-        SELECT id, event, last_receive_id
+        SELECT *
           FROM shelved
-         WHERE timeout!=0 AND (update_time + INTERVAL '1 second' * timeout) < NOW() at time zone 'utc'
-        """
+         WHERE COALESCE(timeout, {timeout})!=0 AND (update_time + INTERVAL '1 second' * timeout) < NOW() at time zone 'utc'
+        """.format(timeout=current_app.config['SHELVE_TIMEOUT'])
         unshelve = self._fetchall(select, {})
 
         # get list of alerts to be unack'ed
         select = """
         WITH acked AS (
-            SELECT DISTINCT ON (a.id) a.id, a.event, a.last_receive_id, h.update_time, a.timeout
+            SELECT DISTINCT ON (a.id) a.*
               FROM alerts a, UNNEST(history) h
              WHERE a.status='ack'
                AND h.type='ack'
                AND h.status='ack'
           ORDER BY a.id, h.update_time DESC
         )
-        SELECT id, event, last_receive_id
+        SELECT *
           FROM acked
-         WHERE timeout!=0 AND (update_time + INTERVAL '1 second' * timeout) < NOW() at time zone 'utc'
-        """
+         WHERE COALESCE(timeout, {timeout})!=0 AND (update_time + INTERVAL '1 second' * timeout) < NOW() at time zone 'utc'
+        """.format(timeout=current_app.config['ACK_TIMEOUT'])
         unack = self._fetchall(select, {})
 
-        return has_expired, unshelve + unack
+        return unshelve + unack
 
     # SQL HELPERS
 
