@@ -66,11 +66,19 @@ class QueryBuilderImpl(QueryBuilder):
         # sort-by
         sort = list()
         direction = 'ASC'
-        if params.get('reverse', None):
+        if params.get('reverse', None):  # deprecated. use '-' instead.
             direction = 'DESC'
         if params.get('sort-by', None):
             for sort_by in params.getlist('sort-by'):
-                if sort_by == 'createTime':
+                direction = 'ASC'
+                if sort_by.startswith('-'):
+                    sort_by = sort_by[1:]
+                    direction = 'DESC'
+                if sort_by == 'severity':
+                    sort.append('s.code ' + direction)
+                elif sort_by == 'status':
+                    sort.append('st.state ' + direction)
+                elif sort_by == 'createTime':
                     sort.append('create_time ' + reverse_sort(direction))
                 elif sort_by == 'receiveTime':
                     sort.append('receive_time ' + reverse_sort(direction))
